<?php
/**
 * Copy a file, or recursively copy a folder and its contents
 * @author      Aidan Lister <aidan@php.net>
 * @version     1.0.1
 * @link        http://aidanlister.com/2004/04/recursively-copying-directories-in-php/
 * @param       string   $source    Source path
 * @param       string   $dest      Destination path
 * @param       int      $permissions New folder creation permissions
 * @return      bool     Returns true on success, false on failure
 */
function xcopy($source, $dest, $permissions = 0755) {
    $sourceHash = hashDirectory($source);
    // Check for symlinks
    if (is_link($source)) {
        return symlink(readlink($source), $dest);
    }

    // Simple copy for a file
    if (is_file($source)) {
        return copy($source, $dest);
    }

    // Make destination directory
    if (!is_dir($dest)) {
        mkdir($dest, $permissions);
    }

    // Loop through the folder
    $dir = dir($source);
    while (false !== $entry = $dir->read()) {
        // Skip pointers
        if ($entry == '.' || $entry == '..') {
            continue;
        }

        // Deep copy directories
        if ($sourceHash != hashDirectory($source . "/" . $entry)) {
            xcopy("$source/$entry", "$dest/$entry", $permissions);
        }
    }

    // Clean up
    $dir->close();
    return true;
}

// In case of coping a directory inside itself, there is a need to hash check the directory otherwise and infinite loop of coping is generated
function hashDirectory($directory) {
    if (!is_dir($directory)) {return false;}

    $files = array();
    $dir = dir($directory);

    while (false !== ($file = $dir->read())) {
        if ($file != '.' and $file != '..') {
            if (is_dir($directory . '/' . $file)) {$files[] = hashDirectory($directory . '/' . $file);} else { $files[] = md5_file($directory . '/' . $file);}
        }
    }

    $dir->close();

    return md5(implode('', $files));
}

function copyDirContentsOnly($source, $dest) {
    foreach (glob($source . DIRECTORY_SEPARATOR . '/{,.}*[!.]', GLOB_MARK | GLOB_BRACE) as $file) {
        if(is_readable($file)) {
            xcopy($source . DIRECTORY_SEPARATOR . basename($file), $dest . DIRECTORY_SEPARATOR . basename($file));
        }
    }    
}
