<?php
ignore_user_abort(true);

// Turn off gzip compression for this response so we can send partial response back (ob_flush())
if (function_exists('apache_setenv')) apache_setenv('no-gzip', 1);
ini_set('zlib.output_compression', 0);

// Turn on output buffering if necessary
if (ob_get_level() == 0) ob_start();

// Remove content encoding like gzip etc.
header('Content-encoding: none', true);

// Return 1x1 pixel transparent gif
header("Content-type: image/gif");

// Don't cache the pixel
header("Content-Length: 42");
header("Cache-Control: private, no-cache, no-cache=Set-Cookie, proxy-revalidate");
header("Expires: Wed, 11 Jan 2000 12:59:00 GMT");
header("Last-Modified: Wed, 11 Jan 2006 12:59:00 GMT");
header("Pragma: no-cache");

// Cross-domain request to allow post
header('Access-Control-Allow-Origin: *');
header('Access-Control-Allow-Methods: POST, GET, OPTIONS');
header('Access-Control-Max-Age: 1000');
header('Access-Control-Allow-Headers: Content-Type');

echo base64_decode('R0lGODlhAQABAID/AMDAwAAAACH5BAEAAAAALAAAAAABA‌​AEAAAICRAEA');

// Flush all output buffers. No reason to make the user wait. 
ob_flush();
flush();
ob_end_flush();

/**
 * UXWizz tracking logic starts here
 */
include '../dbconfig.php';

// Here's the argument from the client.
$clientPageID = $_REQUEST['i']; // clientPageID
$json         = null;

// Update time of latest known activity
// Only if not receiving a full recording, otherwise we might receive very old data and mess up session length
if (!isset($_REQUEST['w'])) {
    $query = "UPDATE `ust_clientpage`, `ust_clients` 
            SET 
                ust_clientpage.last_activity = NOW(),
                ust_clients.last_date = NOW()
            WHERE 
            ust_clientpage.clientid = ust_clients.id AND
            ust_clientpage.id = :clientPageID";
    $stmt = $db->prepare($query);
    $stmt->bindValue(':clientPageID', $clientPageID, PDO::PARAM_INT);
    $stmt->execute();
}

// If $_REQUEST['w'] is set we have a full recording
if (isset($_REQUEST['w']) && isset($_REQUEST['r'])) {
    $json = $_REQUEST['r']; // record
    $recordType = isset($_REQUEST['x']) ? $_REQUEST['x'] : 0;

    try {
        $query = "INSERT INTO `ust_records` (client, record, recordType) VALUES (:clientPageID, :json, :recordType)";
        $stmt = $db->prepare($query);
        $stmt->bindValue(':clientPageID', $clientPageID, PDO::PARAM_INT);
        $stmt->bindValue(':json', $json, PDO::PARAM_STR);
        $stmt->bindValue(':recordType', $recordType, PDO::PARAM_INT);
        $stmt->execute();
    } catch (PDOException $e) {
        die("Could not insert full record into database.\n" . $query);
    }
    die();
}

// Add the partial
if (isset($_REQUEST['p'])) {
    $partial = $_REQUEST['p'];
    $recordType = isset($_REQUEST['x']) ? $_REQUEST['x'] : 0;
    try {
        $query = "INSERT INTO `ust_partials` (client, record, recordType) VALUES (:clientPageID, :json, :recordType)
                    ON DUPLICATE KEY UPDATE record = CONCAT(record, :json2)";
        $stmt = $db->prepare($query);
        $stmt->bindValue(':clientPageID', $clientPageID, PDO::PARAM_INT);
        $stmt->bindValue(':recordType', $recordType, PDO::PARAM_INT);
        $stmt->bindValue(':json', $partial, PDO::PARAM_STR);
        $stmt->bindValue(':json2', $partial, PDO::PARAM_STR);
        $stmt->execute();
    } catch (PDOException $e) {
        echo "Could not insert partial record into database.\n" . $query;
    }
}

// Add the movements
if (isset($_REQUEST['m'])) {
    $movements = $_REQUEST['m'];
    try {
        $query = "INSERT INTO `ust_movements` (client, data) VALUES (:clientPageID, :json)";
        $stmt = $db->prepare($query);
        $stmt->bindValue(':clientPageID', $clientPageID, PDO::PARAM_INT);
        $stmt->bindValue(':json', $movements, PDO::PARAM_STR);
        $stmt->execute();
    } catch (PDOException $e) {
        echo "Could not insert data into database.\n" . $query;
    }
}

// Add the clicks
if (isset($_REQUEST['c'])) {
    $clicks = $_REQUEST['c'];
    try {
        $query = "INSERT INTO `ust_clicks` (client, data) VALUES (:clientPageID, :json)";
        $stmt = $db->prepare($query);
        $stmt->bindValue(':clientPageID', $clientPageID, PDO::PARAM_INT);
        $stmt->bindValue(':json', $clicks, PDO::PARAM_STR);
        $stmt->execute();
    } catch (PDOException $e) {
        echo "Could not insert data into database.\n" . $query;
    }
}
